==> multiphasepy-1.2.0.tar.gz: multiphasepy-1.2.0/multiphasepy/rodare.py <==
# This file is part of the Multiphase Python Repository by HZDR.
#
# Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
# Website: https://hzdr.de
#
# Multiphase Python Repository by HZDR is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Multiphase Python Repository by HZDR is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the Multiphase Python Repository by HZDR. If not, see
# <http://www.gnu.org/licenses/>.

"""Managing software publications in Rossendorf Data Repository (Rodare)."""

import requests
import logging
from datetime import date
from functools import partial
from typing import NoReturn
from sys import exit
from mistune import markdown
from os import path

# Enable logging for this module
log = logging.getLogger(__name__)


def citationcff2rodare(citation_data: dict, pubid: int) -> dict:
    """Transcodes the CITATION.cff schema to Rodare metadata.

    Convert a dictionary obtained from reading a CITATION.cff (yaml) file into
    corresponding metadata required for Rossendorf Data Repository (Rodare).
    Some of the keys and values are just mapped one to one, other like authors
    are converted into the correct format for Rodare. The content of the
    'description' key should be plain text or markdown, and will be converted to
    html. For references only 'generic' types are allowed, and the 'title' key
    is taken as full citation string for the corresponding reference. In
    contrast to codemeta scheme it is not possible to specify either
    contributors (which are not authors) or communities.

    Args:
        citation_data: dictionary with the data obtained from CITATION.cff
        pubid: id for HZDR-internal Robis database.

    Returns:
        A dictionary with the metatdata required for Rodare
    """
    # Mandantory keys
    try:
        creators = []
        for author in citation_data["authors"]:
            _temp = {}
            _temp["name"] = (
                author["family-names"] + ", " + author["given-names"]
            )
            _temp["affiliation"] = author["affiliation"]
            _temp["orcid"] = author["orcid"].replace("https://orcid.org/", "")
            creators.append(_temp)
    except KeyError as e:
        log.error("No author(s) are defined. Use key 'authors'.")
        log.debug(e)
        exit(1)

    try:
        c_name = citation_data["title"]
    except KeyError as e:
        log.error("No name is defined. Use key 'title'.")
        log.debug(e)
        exit(1)

    try:
        c_license_id = citation_data["license"]
    except KeyError as e:
        log.error("No license is defined. Use key 'license'.")
        log.debug(e)
        exit(1)

    try:
        c_description = citation_data["abstract"]
    except KeyError as e:
        log.error("No description is defined. Use key 'abstract'.")
        log.debug(e)
        exit(1)

    try:
        c_keywords = citation_data["keywords"]
    except KeyError as e:
        log.error("No keyword(s) are defined. Use key 'keywords'.")
        log.debug(e)
        exit(1)

    try:
        c_software_version = citation_data["version"]
    except KeyError as e:
        log.error("No version is defined. Use key 'version'.")
        log.debug(e)
        exit(1)

    try:
        c_upload_type = citation_data["type"]
    except KeyError as e:
        log.error("No type is defined. Use key 'type'.")
        log.debug(e)
        exit(1)

    # Optional keys
    try:
        references = []
        for reference in citation_data["references"]:
            if reference["type"] == "generic":
                references.append(reference["title"])
            else:
                log.error(
                    "Wrong reference format. Only type 'generic' with key \
'title' is allowed."
                )
                exit(1)
    except KeyError:
        log.debug("No references are specified.")
        pass

    rodare_metadata = {
        "metadata": {
            "title": c_name,
            "upload_type": c_upload_type,
            "description": markdown(c_description).strip("\n"),
            "language": "eng",
            "license": c_license_id,
            "publication_date": date.today().strftime("%Y-%m-%d"),
            "pub_id": pubid,
            "doc_id": "1",
            "keywords": c_keywords,
            "creators": creators,
            "version": c_software_version,
            "references": references,
        }
    }

    return rodare_metadata


def codemeta2rodare(codemeta_data: dict, pubid: int) -> dict:
    """Transcodes the codemeta schema to Rodare metadata.

    Convert a dictionary obtained from reading a codemeta.json file into
    corresponding metadata required for Rossendorf Data Repository (Rodare).
    Some of the keys and values are just mapped one to one, other like authors
    are converted into the correct format for Rodare. When using codemeta scheme
    it is possible to differentiate between authors and contributors.
    Contributors are relabeled for Rodare to 'Other'. Also it is possible to
    specify communities for Rodare. The content of the 'description' key should
    be valid html or plain text.

    Args:
        codemeta_data: dictionary with the data obtained from codemeta.json
        pubid: id for HZDR-internal Robis database.

    Returns:
        A dictionary with the metatdata required for Rodare
    """
    c_authors = codemeta_data["author"]
    c_contributors = codemeta_data["contributor"]
    c_languages = codemeta_data["programmingLanguage"]
    c_application_categories = codemeta_data["applicationCategory"]
    c_name = codemeta_data["name"]
    c_license_id = codemeta_data["license"]
    c_description = codemeta_data["description"]
    c_keywords = codemeta_data["keywords"]
    c_software_version = codemeta_data["softwareVersion"]
    c_upload_type = codemeta_data["@type"]
    c_references = codemeta_data["referencePublication"]
    c_lang = "eng"  # or deu
    c_publication_date = date.today().strftime("%Y-%m-%d")

    if "Software" in c_upload_type:
        c_upload_type = "software"

    creators = []
    for author in c_authors:
        _temp = {}
        given_name = author["givenName"]
        f_name = author["familyName"]
        affiliation = author["affiliation"]["name"]
        _temp["name"] = f_name + ", " + given_name
        _temp["affiliation"] = affiliation
        if "@id" in author:
            _temp["orcid"] = author["@id"].replace("https://orcid.org/", "")
        creators.append(_temp)

    contributors = []
    for contributor in c_contributors:
        _temp = {}
        given_name = contributor["givenName"]
        f_name = contributor["familyName"]
        affiliation = contributor["affiliation"]["name"]
        _temp["name"] = f_name + ", " + given_name
        _temp["affiliation"] = affiliation
        if "@id" in contributor:
            _temp["orcid"] = contributor["@id"].replace(
                "https://orcid.org/", ""
            )
        _temp["type"] = "Other"
        contributors.append(_temp)

    references = []
    for reference in c_references:
        _name = reference["name"]
        references.append(_name)

    communities = []
    c_application_categories = c_application_categories.split(",")
    for category in c_application_categories:
        _temp = {}
        _temp["identifier"] = category.strip()
        communities.append(_temp)

    rodare_metadata = {
        "metadata": {
            "title": c_name,
            "upload_type": c_upload_type,
            "description": c_description,
            "creators": creators,
            "contributors": contributors,
            "version": c_software_version,
            "keywords": c_keywords + c_languages,
            "references": references,
            "language": c_lang,
            "license": c_license_id,  # for eg. 'GPL-3.0-or-later'
            "publication_date": c_publication_date,
            "pub_id": pubid,
            "doc_id": "1",
            "communities": communities,
        }
    }

    return rodare_metadata


def handle_response(
    res: requests.Response, expected_status_code: int, error_message: str
) -> NoReturn:
    """Handles response and error codes from Rodare API.

    Args:
        res: Response object from request module
        expected_status_code: the expected status code for this request
        error_message: error message shown in case the obtained status code
                       does not match the expected status code

    Returns:
        Nothing.
    """
    if res.status_code == expected_status_code:
        return

    # Print status code and user-supplied error message
    log.error(f"Status code {res.status_code}: {error_message}")

    try:
        # See if the server replied with a JSON object
        content = res.json()
        # If the the res has a message field, print it
        if (message := content.get("message")) is not None:
            log.error(f"Message: {message}")
        # If the res has an errors field, print them all (e.g. multiple
        # errors when metadata validation fails)
        if errors := content.get("errors"):
            log.error("Errors:")
            for error in errors:
                log.error(f"  {error}")
    except requests.exceptions.JSONDecodeError:
        # Server did not reply with valid JSON so we print the res as text
        log.error(f"Server response: {res.text}")

    exit(1)


def publish_version(
    url: str, token: str, id: int, files: list[str], rodare_data: dict
) -> NoReturn:
    """Publish a new version of a software package in Rodare.

    Get the latest version from Rodare for the software package, create a new
    bucket for a new version, upload all files, update metadata and publish
    the new version.

    Args:
        url: URL of Rodare API
        token: token to access Rodare API
        id: id of the software publication in Rodare
        files: file(s) to upload for the software publication
        rodare_data: dictionary with metadata for the new version

    Returns:
        Nothing.
    """
    session = requests.Session()
    session.headers = {"Authorization": f"Bearer {token}"}

    # Globally set timeout by shimming request method
    session.request = partial(session.request, timeout=10)

    # Get deposition metadata
    response = session.get(f"{url}/{id}")
    handle_response(
        response, 200, f"Could not retrieve deposition resource for {id}."
    )

    links = response.json()["links"]
    newversion_url = links["newversion"]
    latest_release_url = links["latest"]

    # Save related identifiers from latest release
    response = session.get(latest_release_url)
    handle_response(response, 200, "Could not retrieve latest release info.")
    related_identifiers = None
    if metadata := response.json().get("metadata"):
        related_identifiers = metadata.get("related_identifiers")
        # Filter out "isVersionOf" relation which points to Concept ID
        related_identifiers = list(
            filter(
                lambda id: id.get("relation") != "isVersionOf",
                related_identifiers,
            )
        )

    # Create a new version
    response = session.post(newversion_url)
    handle_response(response, 201, "Could not create new version.")
    latest_draft_url = response.json()["links"]["latest_draft"]

    # Get some urls for new version
    response = session.get(latest_draft_url)
    handle_response(response, 200, "Could not get latest draft.")
    links = response.json()["links"]
    files_url = links["files"]
    bucket_url = links["bucket"]
    publish_url = links["publish"]
    deposit_url = links["self"]

    # Clean files
    response = session.get(files_url)
    for file in response.json():
        response = session.delete(file["links"]["download"])
        handle_response(response, 204, f"Unable to delete {file['filename']}.")

    # Upload files
    for file in files:
        with open(file, "rb") as file_data:
            response = session.put(
                f"{bucket_url}/{path.basename(file)}", data=file_data
            )
            handle_response(response, 200, f"Unable to upload file {file}.")

    if related_identifiers is not None:
        rodare_data["metadata"]["related_identifiers"] = related_identifiers
    response = session.put(deposit_url, json=rodare_data)
    handle_response(response, 200, "Metadata cannot be set.")

    # Publish record
    response = session.post(publish_url)
    handle_response(response, 202, "Cannot publish record.")

==> multiphasepy-1.4.5.tar.gz: multiphasepy-1.4.5/multiphasepy/rodare.py <==
# This file is part of the Multiphase Python Repository by HZDR.
#
# Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
# Website: https://hzdr.de
#
# Multiphase Python Repository by HZDR is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Multiphase Python Repository by HZDR is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the Multiphase Python Repository by HZDR. If not, see
# <http://www.gnu.org/licenses/>.

"""Managing software publications in Rossendorf Data Repository (Rodare)."""

import logging
from datetime import date
from functools import partial
from os import path
from sys import exit
from typing import NoReturn

import requests
from mistune import markdown

# Enable logging for this module
log = logging.getLogger(__name__)


def citationcff2rodare(citation_data: dict, pubid: int) -> dict:
    """Transcodes the CITATION.cff schema to Rodare metadata.

    Convert a dictionary obtained from reading a CITATION.cff (yaml) file into
    corresponding metadata required for Rossendorf Data Repository (Rodare).
    Some of the keys and values are just mapped one to one, other like authors
    are converted into the correct format for Rodare. The content of the
    'description' key should be plain text or markdown, and will be converted to
    html. For references only 'generic' types are allowed, and the 'title' key
    is taken as full citation string for the corresponding reference. In
    contrast to codemeta scheme it is not possible to specify either
    contributors (which are not authors) or communities.

    Args:
        citation_data: dictionary with the data obtained from CITATION.cff
        pubid: id for HZDR-internal Robis database.

    Returns:
        A dictionary with the metadata required for Rodare
    """
    # Mandantory keys
    try:
        creators = []
        for author in citation_data["authors"]:
            _temp = {}
            _temp["name"] = (
                author["family-names"] + ", " + author["given-names"]
            )
            _temp["affiliation"] = author["affiliation"]
            _temp["orcid"] = author["orcid"].replace("https://orcid.org/", "")
            creators.append(_temp)
        c_name = citation_data["title"]
        c_license_id = citation_data["license"]
        c_description = citation_data["abstract"]
        c_keywords = citation_data["keywords"]
        c_software_version = citation_data["version"]
        c_upload_type = citation_data["type"]
    except KeyError as e:
        log.error(f"The following key is mandatory: {e}.")
        log.debug(e)
        exit(1)

    # Optional keys
    try:
        references = []
        for reference in citation_data["references"]:
            if reference["type"] == "generic":
                references.append(reference["title"])
            else:
                log.error(
                    "Wrong reference format. Only type 'generic' with key \
'title' is allowed."
                )
                exit(1)
    except KeyError as e:
        log.debug(f"Optional key not found: {e}.")
        pass

    rodare_metadata = {
        "metadata": {
            "title": c_name,
            "upload_type": c_upload_type,
            "description": markdown(c_description).strip("\n"),
            "language": "eng",
            "license": c_license_id,
            "publication_date": date.today().strftime("%Y-%m-%d"),
            "pub_id": pubid,
            "doc_id": "1",
            "keywords": c_keywords,
            "creators": creators,
            "version": c_software_version,
            "references": references,
        }
    }

    return rodare_metadata


def codemeta2rodare(codemeta_data: dict, pubid: int) -> dict:
    """Transcodes the codemeta schema to Rodare metadata.

    Convert a dictionary obtained from reading a codemeta.json file into
    corresponding metadata required for Rossendorf Data Repository (Rodare).
    Some of the keys and values are just mapped one to one, other like authors
    are converted into the correct format for Rodare. When using codemeta scheme
    it is possible to differentiate between authors and contributors.
    Contributors are relabeled for Rodare to 'Other'. Also it is possible to
    specify communities for Rodare. The content of the 'description' key should
    be valid html or plain text.

    Args:
        codemeta_data: dictionary with the data obtained from codemeta.json
        pubid: id for HZDR-internal Robis database.

    Returns:
        A dictionary with the metadata required for Rodare
    """
    # Mandatory information
    try:
        c_authors = codemeta_data["author"]
        c_contributors = codemeta_data["contributor"]
        c_languages = codemeta_data["programmingLanguage"]
        c_application_categories = codemeta_data["applicationCategory"]
        c_name = codemeta_data["name"]
        c_license_id = codemeta_data["license"]
        c_description = codemeta_data["description"]
        c_keywords = codemeta_data["keywords"]
        c_software_version = codemeta_data["softwareVersion"]
        c_upload_type = codemeta_data["@type"]
        c_lang = "eng"  # or deu
        c_publication_date = date.today().strftime("%Y-%m-%d")
    except KeyError as e:
        log.error(f"The following entry is mandatory: {e}.")
        exit(1)

    # Optional keys
    try:
        c_references = codemeta_data["referencePublication"]
    except KeyError as e:
        c_references = []
        log.debug(f"Optional key not found: {e}.")
        pass

    if "Software" in c_upload_type:
        c_upload_type = "software"

    creators = []
    for author in c_authors:
        _temp = {}
        given_name = author["givenName"]
        f_name = author["familyName"]
        affiliation = author["affiliation"]["name"]
        _temp["name"] = f_name + ", " + given_name
        _temp["affiliation"] = affiliation
        if "@id" in author:
            _temp["orcid"] = author["@id"].replace("https://orcid.org/", "")
        creators.append(_temp)

    contributors = []
    for contributor in c_contributors:
        _temp = {}
        given_name = contributor["givenName"]
        f_name = contributor["familyName"]
        affiliation = contributor["affiliation"]["name"]
        _temp["name"] = f_name + ", " + given_name
        _temp["affiliation"] = affiliation
        if "@id" in contributor:
            _temp["orcid"] = contributor["@id"].replace(
                "https://orcid.org/", ""
            )
        _temp["type"] = "Other"
        contributors.append(_temp)

    references = []
    for reference in c_references:
        _name = reference["name"]
        references.append(_name)

    communities = []
    c_application_categories = c_application_categories.split(",")
    for category in c_application_categories:
        _temp = {}
        _temp["identifier"] = category.strip()
        communities.append(_temp)

    rodare_metadata = {
        "metadata": {
            "title": c_name,
            "upload_type": c_upload_type,
            "description": c_description,
            "creators": creators,
            "contributors": contributors,
            "version": c_software_version,
            "keywords": c_keywords + c_languages,
            "references": references,
            "language": c_lang,
            "license": c_license_id,  # for eg. 'GPL-3.0-or-later'
            "publication_date": c_publication_date,
            "pub_id": pubid,
            "doc_id": "1",
            "communities": communities,
        }
    }

    return rodare_metadata


def handle_response(
    res: requests.Response, expected_status_code: int, error_message: str
) -> NoReturn:
    """Handles response and error codes from Rodare API.

    Args:
        res: Response object from request module
        expected_status_code: the expected status code for this request
        error_message: error message shown in case the obtained status code
                       does not match the expected status code

    Returns:
        Nothing.
    """
    if res.status_code == expected_status_code:
        return

    # Print status code and user-supplied error message
    log.error(f"Status code {res.status_code}: {error_message}")

    try:
        # See if the server replied with a JSON object
        content = res.json()
        # If the the res has a message field, print it
        if (message := content.get("message")) is not None:
            log.error(f"Message: {message}")
        # If the res has an errors field, print them all (e.g. multiple
        # errors when metadata validation fails)
        if errors := content.get("errors"):
            log.error("Errors:")
            for error in errors:
                log.error(f"  {error}")
    except requests.exceptions.JSONDecodeError:
        # Server did not reply with valid JSON so we print the res as text
        log.error(f"Server response: {res.text}")

    exit(1)


def publish_version(
    url: str, token: str, id: int, files: list[str], rodare_data: dict
) -> NoReturn:
    """Publish a new version of a software package in Rodare.

    Get the latest version from Rodare for the software package, create a new
    bucket for a new version, upload all files, update metadata and publish
    the new version.

    Args:
        url: URL of Rodare API
        token: token to access Rodare API
        id: id of the software publication in Rodare
        files: file(s) to upload for the software publication
        rodare_data: dictionary with metadata for the new version

    Returns:
        Nothing.
    """
    session = requests.Session()
    session.headers = {"Authorization": f"Bearer {token}"}

    # Globally set timeout by shimming request method
    session.request = partial(session.request, timeout=10)

    # Get deposition metadata
    response = session.get(f"{url}/{id}")
    handle_response(
        response, 200, f"Could not retrieve deposition resource for {id}."
    )

    links = response.json()["links"]
    newversion_url = links["newversion"]
    latest_release_url = links["latest"]

    # Save related identifiers from latest release
    response = session.get(latest_release_url)
    handle_response(response, 200, "Could not retrieve latest release info.")
    related_identifiers = None
    if metadata := response.json().get("metadata"):
        related_identifiers = metadata.get("related_identifiers")
        # Filter out "isVersionOf" relation which points to Concept ID
        related_identifiers = list(
            filter(
                lambda id: id.get("relation") != "isVersionOf",
                related_identifiers,
            )
        )

    # Create a new version
    response = session.post(newversion_url)
    handle_response(response, 201, "Could not create new version.")
    latest_draft_url = response.json()["links"]["latest_draft"]

    # Get some urls for new version
    response = session.get(latest_draft_url)
    handle_response(response, 200, "Could not get latest draft.")
    links = response.json()["links"]
    files_url = links["files"]
    bucket_url = links["bucket"]
    publish_url = links["publish"]
    deposit_url = links["self"]

    # Clean files
    response = session.get(files_url)
    for file in response.json():
        response = session.delete(file["links"]["download"])
        handle_response(response, 204, f"Unable to delete {file['filename']}.")

    # Upload files
    for file in files:
        with open(file, "rb") as file_data:
            response = session.put(
                f"{bucket_url}/{path.basename(file)}", data=file_data
            )
            handle_response(response, 200, f"Unable to upload file {file}.")

    if related_identifiers is not None:
        rodare_data["metadata"]["related_identifiers"] = related_identifiers
    response = session.put(deposit_url, json=rodare_data)
    handle_response(response, 200, "Metadata cannot be set.")

    # Publish record
    response = session.post(publish_url)
    handle_response(response, 202, "Cannot publish record.")
